
/* BALLISTIC CHRONOGRAPH
 *** NOT FOR DISTRIBUTION ***
  20 character 4 line I2C Liquid Crystal Display
  Backpack Interface labelled "YwRobot Arduino LCM1602 IIC V1"
  Connect Vcc and Ground, SDA to A4, SCL to A5 on Arduino Nano
  Connect the menu UP button to pin A1
  Connect the menu DOWN button to pin A0
  First IR sensor (OPL550) input pin D10
  SecondIR sensor (OPL550) input pin D11

  ***ENSURE CORRECT SENSOR SEPARATION DISTANCE IS SELECTED BELOW (line 177 *********)
    ENSURE THAT ALL LIBRARIES ARE DOWNLOADED TO YOUR SYSTEM FIRST
  John W 16/06/2021  trailcam99@gmail.com
*/

#include <Wire.h>
#include <EEPROM.h>
#include <hd44780.h>                       // main hd44780 header library
#include <hd44780ioClass/hd44780_I2Cexp.h> // i2c expander i/o class header

hd44780_I2Cexp lcd; // declare lcd object: auto locate & config expander chip

#define firstsensor_1 10   //D10 IR sensor (OPL550) input pin
#define secondsensor_2 11  //D11 IR sensor (OPL550)input pin
#define UP A1    //Set the menu up button to input A1
#define DOWN A0  //Set the menu down button to input A0

float fps = 0; //storage for feet per sec value
float ftlbs = 0; //storage for pellet energy value
float pWeight; //Pellet weight in grains
float elap, ms, joule;
float fpstotal = 0;
float average = 0;
float ftlbstotal = 0;
float energy = 0;

const int LCD_COLS = 20;  // LCD geometry
const int LCD_ROWS = 4;   // LCD geometry

int weightTens; //first two digits of pWeight
int weightDec; //Decimal part of Pellet weight
int one = 1;   //pellet weight loop control
int counter_shots = 0; //increment after each shot

unsigned long time1;
unsigned long time2;

void setup() {

  //Serial.begin(9600);
  lcd.begin(LCD_COLS, LCD_ROWS);

  weightTens = EEPROM.read(0); // retrieve pellet weight
  weightDec = EEPROM.read(1);
  pWeight = weightTens + float(weightDec) / 100;

  pinMode(UP, INPUT);
  pinMode(DOWN, INPUT);
  digitalWrite (UP, HIGH);  //turn on pullup resistor
  digitalWrite (DOWN, HIGH);

  pinMode (firstsensor_1, INPUT);
  pinMode (secondsensor_2, INPUT);
  digitalWrite(firstsensor_1, HIGH);
  digitalWrite(secondsensor_2, HIGH);
  reset_variables();

  lcd.setCursor(3, 0);  //Start at character 3 on line 0 (*****character 0 is the first(LH)character****line 0 is TOP LINE****)
  lcd.print("Chronograph V8.3");
  delay(2000);
}

void loop()
{
  if (one == 1)
  {
    firstTime ();
  }
  everyTime();
}

void firstTime() //this is pellet weight code, runs ONCE only
//**** WHEN 'UP' BUTTON IS HELD DOWN AT SWITCH-ON********

{
  one = 2;

  lcd.setCursor(1, 0);   //Start at character 1 on line 0
  lcd.print("Enter pellet weight");
  lcd.setCursor(4, 2);
  lcd.print(pWeight);
  lcd.setCursor(9, 2);  //Start at character 9 on line 2
  lcd.print("grains");
  lcd.setCursor(3, 3);
  delay(2000);

  while (digitalRead(UP) == LOW )
  {
    pWeight = pWeight + 0.01 ;
    lcd.setCursor(4, 2);
    lcd.print(pWeight);
    weightDec = (pWeight - int(pWeight)) * 100;
    delay(300);
    EEPROM.write(0, int(pWeight));
    EEPROM.write(1, weightDec);

  }
  delay(2000);
  while (digitalRead(DOWN) == LOW )
  {
    pWeight = pWeight - 0.01 ;
    lcd.setCursor(4, 2);
    lcd.print(pWeight);
    weightDec = (pWeight - int(pWeight)) * 100;
    delay(300);
    EEPROM.write(0, int(pWeight));
    EEPROM.write(1, weightDec);

  }
  delay(2000);
  while (digitalRead(UP) == LOW )    // repeats button read sequence again
  {
    pWeight = pWeight + 0.01 ;
    lcd.setCursor(4, 2);
    lcd.print(pWeight);
    weightDec = (pWeight - int(pWeight)) * 100;
    delay(300);
    EEPROM.write(0, int(pWeight));
    EEPROM.write(1, weightDec);

  }
  delay(2000);
  while (digitalRead(DOWN) == LOW )
  {
    pWeight = pWeight - 0.01 ;
    lcd.setCursor(4, 2);
    lcd.print(pWeight);
    weightDec = (pWeight - int(pWeight)) * 100;
    delay(300);
    EEPROM.write(0, int(pWeight));
    EEPROM.write(1, weightDec);

  }
  delay(1000);
}

void everyTime()  // this is pellet timing code loops everytime

{
  lcd.setCursor(3, 3);
  lcd.print("Awaiting shot");
  lcd.blink();       //display flashing cursor

  while (digitalRead(firstsensor_1) == 0);  //pellet timing
  while (digitalRead(firstsensor_1));
  time1 = micros();

  while (digitalRead(secondsensor_2));
  time2 = micros();

  printserial();

}       //****end of timing loop*****

void printserial()
{

  lcd.clear();
  lcd.setCursor(3, 0);
  lcd.print("Shot number: ");
  delay(300);
  lcd.print(++counter_shots);

  elap = time2 - time1;

  fps = 196850 / elap;      // for 60mm sensor spacing distance   **********************
  //fps = 262467 / elap;      // for 80mm sensor spacing distance **********************
  //fps = 328084 / elap;      // for 100mm sensor spacing distance **********************

  // Serial.print("FPS\t");
  delay(300);
  lcd.setCursor(0, 1);   //Start at character 0 on line 1
  lcd.print(fps );
  lcd.setCursor(6, 1);
  lcd.print("fps");

  delay(300);
  ms = fps * 0.3048;  //metres per second calculation
  lcd.setCursor(11, 1);  //Start at character 11 on line 1
  lcd.print(ms );

  lcd.setCursor(17, 1);
  lcd.print("mps");

  delay(300);

  ftlbs = (pWeight * fps * fps) / 450240;  //foot pounds and joules energy calculations

  joule = ftlbs * 1.35582;

  lcd.setCursor(1, 2);    //Start at character 1 on line 2
  lcd.print("Ft/Lbs:");
  delay(300);
  //Serial.print("Ft/Lbs\t");
  //Serial.println(ftlbs );
  lcd.print(ftlbs );

  lcd.setCursor(1, 3);
  lcd.print("Joules:");
  delay(300);
  lcd.print(joule );

  delay(5000);
  lcd.clear();


  fpstotal = fpstotal + fps;  // averaging code after 5 shots
  average = fpstotal / (counter_shots);
  ftlbstotal = ftlbstotal + ftlbs;
  energy = ftlbstotal / (counter_shots);

  if (counter_shots == 5) {
    lcd.setCursor(4, 0);    //Start at character 4 on line 0
    lcd.print ("Shot Average");
    lcd.setCursor(0, 1);
    lcd.print ("Velocity:");
    lcd.print(average);
    lcd.setCursor(17, 1);
    lcd.print ("FPS");

    lcd.setCursor(0, 2);
    lcd.print ("Energy:");
    lcd.print(energy);
    lcd.setCursor(14, 2);
    lcd.print ("Ft/Lbs");

  }

  if (counter_shots == 10) {   // repeat averaging code after 10 shots!
    lcd.setCursor(4, 0);    //Start at character 4 on line 0
    lcd.print ("Shot Average");
    lcd.setCursor(0, 1);
    lcd.print ("Velocity:");
    lcd.print(average);
    lcd.setCursor(17, 1);
    lcd.print ("FPS");

    lcd.setCursor(0, 2);
    lcd.print ("Energy:");
    lcd.print(energy);
    lcd.setCursor(14, 2);
    lcd.print ("Ft/Lbs");

  }


}

void reset_variables() {
  time1 = 0;
  time2 = 0;
}
// HOPE YOU HAVE FUN USING THIS ! CONTACT EMAIL ABOVE FOR QUERIES
